<?php

namespace Drupal\sidekick;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\Core\Render\Renderer;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\BadResponseException;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Exception\TransferException;
use GuzzleHttp\RequestOptions;

/**
 * Service for HTTP request.
 */
class SidekickService
{

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $http_client;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The sidekick config.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $configFactory;

  /**
   * Language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * @var string
   */
  protected $key_status_api;

  /**
   * @var string
   */
  protected $account_api;

  /**
   * @param \GuzzleHttp\ClientInterface $http_client
   * The Http client
   * @param \Drupal\Core\Session\AccountInterface $currentUser
   * The current user.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   * The configuration factory.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   * LanguageManager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   * The entity type manager.
   * @param \Drupal\Core\Render\Renderer $renderer
   * Renderer service.
   */
  public function __construct(
    ClientInterface $http_client,
    AccountInterface $currentUser,
    ConfigFactoryInterface $config_factory,
    LanguageManagerInterface $languageManager,
    EntityTypeManagerInterface $entity_type_manager,
    Renderer $renderer
  ) {
    $this->http_client = $http_client;
    $this->currentUser = $currentUser;
    $this->config_factory = $config_factory->get('sidekick.settings');
    $this->languageManager = $languageManager;
    $this->entityTypeManager = $entity_type_manager;
    $this->renderer = $renderer;
    $this->key_status_api = 'https://assistant.ai-sidekick.app/api/v1/apikey-status';
    $this->account_api = 'https://assistant.ai-sidekick.app/api/v1/accounts/current';
  }

  /**
   * @param mixed $api_key
   * @param mixed $language_code
   *
   * @return array|mixed
   */
  public function checkKeyStatus($api_key, $language_code)
  {
    try {
      $response = $this->http_client->post($this->key_status_api, [
        'headers' => [
          'Content-Type' => 'application/json',
        ],
        'body' => json_encode([
          'token' => $api_key,
          'language' => $language_code,
        ]),
      ]);

      $statusCode = $response->getStatusCode();

      return [
        'status' => $statusCode,
        'message' => json_decode($response->getBody()),
      ];
    } catch (ClientException|RequestException|TransferException|BadResponseException $exception) {
      watchdog_exception('sidekick', $exception, null, [], 6);

      return json_decode((string)$exception->getResponse()->getBody());
    }
  }

  /**
   * @param $api_key
   * @return array|mixed
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function getAccountContext($api_key)
  {
    try {
      $request_options = [];
      $request_options[RequestOptions::HEADERS] = [
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $api_key
      ];
      $response = $this->http_client->request(
        'GET',
        $this->account_api,
        $request_options
      );
      $statusCode = $response->getStatusCode();
      return [
        'status' => $statusCode,
        'message' => Json::decode($response->getBody(), true),
      ];
    } catch (ClientException|RequestException|TransferException|BadResponseException $exception) {
      watchdog_exception('sidekick', $exception, null, [], 6);

      return Json::decode((string)$exception->getResponse()->getBody(), true);
    }
  }

  /**
   * @param $api_key
   * @param $page_briefing
   * @param $target_audience
   * @return array|mixed
   * @throws \GuzzleHttp\Exception\GuzzleException
   */
  public function setAccountContext($api_key, $page_briefing, $target_audience)
  {
    try {
      $request_options = [];
      $request_options[RequestOptions::HEADERS] = [
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $api_key
      ];
      $request_options[RequestOptions::BODY] = Json::encode([
        'context' => $page_briefing,
        'target_audience' => $target_audience,
        "process_web_page_context" => true
      ]);
      $response = $this->http_client->request(
        'put',
        $this->account_api,
        $request_options
      );
      $statusCode = $response->getStatusCode();
      return [
        'status' => $statusCode,
        'message' => Json::decode($response->getBody()->getContents(), true),
      ];
    } catch (ClientException|RequestException|TransferException|BadResponseException $exception) {
      watchdog_exception('sidekick', $exception, null, [], 6);

      return Json::decode((string)$exception->getResponse()->getBody(), true);
    }
  }

  /**
   * @param $node
   * @param $form_state
   *
   * @return false|string
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getSideKickMessagingApiData($node, $form_state)
  {
    $nodeUrl = Url::fromRoute(
      'entity.node.canonical',
      ['node' => $node->id()],
      ['absolute' => true]
    )->toString();

    $entity_type = $form_state->getFormObject()->getEntity()
      ->getEntityTypeId();
    $node = $this->entityTypeManager->getStorage($entity_type)
      ->load($node->id());
    $builder = $this->entityTypeManager->getViewBuilder($entity_type);
    $build = $builder->view($node, 'default');
    $output = $this->renderer->render($build);

    $form_fields = $form_state->getFormObject()->getEntity()
      ->getFields();
    foreach ($form_fields as $key => $field) {
      if ($form_state->getFormObject()->getEntity()
        ->get($key)->entity) {
        $structuredContent[$key] = $form_state->getFormObject()
          ->getEntity()->get($key)->entity;
      } else {
        $structuredContent[$key] = $form_state->getFormObject()
          ->getEntity()->get($key)->value;
      }
    }
    //dump($structuredContent); //TODO

    $sideKickMessagingApiData = [
      'version' => '1.0',
      'eventName' => 'page-changed',
      'data' => [
        'url' => $nodeUrl,
        'title' => $node->getTitle(),
        'content' => $output,
        'structuredContent' => [],
        'targetAudience' => 'Tech-savvy people',
        'pageBriefing' => '',
      ],
    ];
    return json_encode($sideKickMessagingApiData);
  }

  public function getQueryParameter($language_code)
  {
    $domain = Url::fromRoute(
      '<front>',
      [],
      ['absolute' => true]
    )->toString();

    return [
      'contentLanguage' => $language_code,
      'interfaceLanguage' => $language_code,
      'language' => $language_code,
      'userId' => $this->currentUser->getDisplayName(),
      'domain' => $domain,
      'apikey' => $this->config_factory->get('api_key'),
      'platform' => 'Drupal',
      'hide_account_context' => true,
      'referral' => 'Acolono',
    ];
  }

}
